package Text::Markdown::Parser;
use strict;
use warnings;
#use Data::Dumper;
use HTML::Entities;
=encoding utf8

=head1 NAME

Text::Markdown::Parser -- parse Markdown

=head1 SYNOPSIS

 use strict;
 use warnings;
 use Text::Markdown::Parser;

 my $parse=Text::Markdown::Parser->new(grep_exessive=>0);
 my $end=0;
 my $deep=0;
 $parse->add_handler('start',sub{
     print "\n"," "x$deep,'>>>',shift(),"\n";
     $deep+=2;
     $end=1;
     return 1;
   },
 'type');

 $parse->add_handler('text',sub{
     my $txt=shift;
     $txt=~s/\n/"\n".(" "x$deep).'|'/gse;
     if($end)
     {
       $txt=(" "x$deep).'|'.$txt;
       $end=0;
     }
     print $txt;
     return 1;
   },
 'text');

 $parse->add_handler('end',sub{
     my $type=shift;
     my $text=shift;
     if($text && ref($text))
     {
       $text="(".join('|',@$text).")";
       $text=~s/\n/\\n/gs;
     }
     else
     { $text='' }
     $deep-=2;
     $end=1;
     print "\n"," "x$deep,"<<<$type $text\n";
     return 1;
   },'type,text');

 $parse->parse(<<'EOT');
 AT&T has an ampersand in their name.
 =====================================

 AT&amp;T is another way to write it.
 ------------------------------------
 EOT

=head1 DESCRIPTION

Parser for markdown formated Documents

=head1 AUTHOR

ToPeG

=head1 METHODS

=over 4

=cut

# CHANGELOG:
#
# 0.1 initial Release

our $VERSION='0.1';

my $grep_link_def=qr/\[(.+?)\]:\s*(\S+)(?:\s*(".*"|'.*'|\(.*\))(?=\s*(?:\n|$))|(?:\s*("[^"]*"|'[^']*'|\([^()]*\)))?)/;

=item new(%options)

create a new Parser Object

returns a Text::Markdown::Parser Object

=over 2

 tabwidth => <int> (default 4)

set the amount of spaces for one tab

 grep_exessive => 1/0 (default 0)

if this option is true, linlinehtml will gab as mutch as possible text

=back

=cut

sub new
{
    my $class = shift;
    my $self = bless {}, ref($class) || $class;
    return $self->init(@_);
}

sub init
{
  my $self = shift;
  pop(@_) if(@_%2);
  my %arg = @_;

  $self->{tabwidth}=4;
  $self->{utf8}=0;
  $self->{grep_exessive}=0;

  if (my $h = delete $arg{handlers})
  {
    $h = {@$h} if ref($h) eq "ARRAY";
    while (my($event, $cb) = each %$h)
    { $self->add_handler($event => @$cb); }
  }

  # In the end we try to assume plain attribute or handler
  while (my($option, $val) = each %arg)
  {
    if ($option =~ /^(\w+)_h$/)
    { $self->add_handler($1 => @$val); }
    elsif(exists($self->{$option}))
    { $self->{$option}=$val; }
  }

  $self->{deep}=[];
  $self->{quotes}={};
  $self->{quote_count}=-1;
  $self->{link_defs}={};

  return $self;
}

=item parse_file(file|filehandle)

parse data from a File or Filehandle

=cut

sub parse_file
{
  my($self, $file) = @_;
  if (!ref($file) && ref(\$file) ne "GLOB")
  {
    open( $file , "<", $file) || return 0;
    binmode($file);
  }

  local $/=undef;
  return $self->parse(<$file>);
}

# set up method stubs
sub text { }
*start = \&text;
*end   = \&text;

=item parse($text)

parse text

=cut

sub parse
{
  my $self=shift;
  my $data=shift;
  $data='' unless(defined($data));
  $data=decode_entities($data);
  $data=~s/\x0D?\x0A|\x0D/\n/gs;
  $data=~s/\t/' ' x $self->{tabwidth}/gse;
  $data=~s/\\([\\`*_{}\[\]()#.!:])/$self->{quote_count}++;$self->{quotes}->{"&quote$self->{quote_count};"}=$1;$_="&quote$self->{quote_count};"/gse;


  # filter all linkdefs:
  my $md=$self->{tabwidth};
  my $pos=0;
  $self->{link_defs}={};
  while($data=~m/$grep_link_def/gc)
  {
    my $bevor=substr($data,$pos,$-[0]-$pos);
    $pos=$+[0];
    # no code matches (this is a hack)
    # I have to full parse the document twice...
    # Idee:
    # Schalter einbauen um bestimmte "block/span" - Elemente zu ignorieren
    # das sollte das Parsen etwas beschleubigen...
    # dafür muss ich aber eine zweite Instanz erzeugen,
    # das nur die Link-Definitionen ausgibt.
    # wird trotzdem langsam sein...
    unless($bevor=~/(^|\n|>) {$md}$/)
    {
      my $id=$1;
      my $url=$2;
      my $title=$3||$id;
      $id=~s/\s+/ /gs;
      unless($title=~s/^(["'])(.+?)\1$/$2/)
      { $title=~s/^\((.+?)\)$/$1/; }
      $self->{link_defs}->{$id}={url=>$url,title=>$title};
    }
  }

  $self->_make_link_def_regexp();

  $self->_do_handler('start',type=>"page");
  $data=$self->_do_parse($data,'');
  $self->_do_handler('end',type=>"page") if($self->_is_inside('page'));
}

my @determin_block=(
    # page end
    qr/\n{2,}/,
    # ol
    qr/\n\d+\./,
    # html
    qr/\n *<\w+/,
    # quote/ul/hr/h1..6
    qr/(?:\n[\-*>+ #]|$)/,
    # h1/h2
    qr/\n[^\n]\n[-=]+/,
    # last block (span)
    qr/[^\n]+$/s,
  );

my @parse_block=(
    {
      name  => 'HTML',
      run   => sub{
          my $self=shift;
          my $data=shift;
          return 0 unless($$data=~m#^ *<\w+(?:\s+\w+=(?:"[^"]*"|'[^']*'|\S+))*\s*/?>#s);
          my $html='';
          ($$data,$html)=$self->_cutout_html($$data);
          $html=$self->_restore_quote($html);
          my $restart_page=0;
          if($self->_is_inside() eq 'page')
          {
            $self->_do_handler('end',type=>"page");
            $restart_page=1;
          }
          $self->_do_handler('start',type=>"html");
          $self->_do_handler('text',text=>$html);
          $self->_do_handler('end',type=>"html");
          $self->_do_handler('start',type=>"page") if($restart_page);
          return 1;
        },
    },
    {
      name  => 'h1 .. h2 Setext',
      run   => sub{
          my $self=shift;
          my $data=shift;
          return 1 unless($$data =~s/^ *([^\n]+?) *\n *(=+|\-+) *(?:\n|$)//);
          my $txt=$1;
          my $type='h1';
          $type='h2' if(index($2,'-')>=0);
          $txt=~s/^\s+//;
          $txt=~s/\s+$//;
          $self->_do_handler('end',type=>"page");
          $self->_do_handler('start',type=>$type);
          $self->_do_parse($txt);
          $self->_do_handler('end',type=>$type);
          $self->_do_handler('start',type=>"page");
        },
    },
    {
      name  => 'h1 .. h6 atx',
      run   => sub{
          my $self=shift;
          my $data=shift;
          return 0 unless ($$data=~s/^ *(#+) *(.+?) *#* *(?:\n|$)//);
          my $p=length($1);
          $p=6 if($p>6);
          $self->_do_handler('end',type=>"page");
          $self->_do_handler('start',type=>"h$p");
          $self->_do_parse($2);
          $self->_do_handler('end',type=>"h$p");
          $self->_do_handler('start',type=>"page");
        },
    },
    {
      name  => 'quote',
      run   => sub{
          my $self=shift;
          my $data=shift;
          return 0 unless ($$data=~/^>( *)/);
          my $l=length($1);
          $self->_do_handler('end',type=>"page");
          $self->_do_handler('start',type=>"quote");
          $self->_do_handler('start',type=>"page");
          my $text='';
          while($$data ne '')
          {
            my $pos=index($$data,"\n");
            if($pos>0)
            {
              my $line=substr($$data,0,$pos+1,'');
              $line=~s/^> {0,$l}//;
              $text.=$line;
            }
            elsif($pos==0)
            {
              substr($$data,0,1,'');
              last;
            }
            else
            {
              $text.=$$data;
              $$data='';
              last;
            }
          }
          $self->_do_parse($text);
          $self->_do_handler('end',type=>"page");
          $self->_do_handler('end',type=>"quote");
          $self->_do_handler('start',type=>"page");
        },
    },
    {
      name  => 'code',
      run   => sub{
          my $self=shift;
          my $data=shift;
          my $sin=$self->{tabwidth}||4;
          return 0 unless($data=~/^( {$sin,})/);
          my $s=$1;
          my $code='';
          $code.="$1\n" while($$data=~s/^$s(.*)(\n|$)//);
          $code=$self->_restore_quote($code);
          $self->_do_handler('start',type=>"code");
          $self->_do_handler('text',text=>$code);
          $self->_do_handler('end',type=>"code");
        },
    },
    {
      name  => 'hr',
      run   => sub{
          my $self=shift;
          my $data=shift;
          my $return=shift;
          return 0 unless($data=~s/^(?:\- ?){3,}\s*(?:\n|$)// || $data=~s/^(?:\* ?){3,}\s*(?:\n|$)//);
          $self->_do_handler('end',type=>"page");
          $self->_do_handler('start',type=>"hr");
          $self->_do_handler('end',type=>"hr");
          $self->_do_handler('start',type=>"page");
        },
    },
    {
      name  => 'list',
      run   => sub{
          my $self=shift;
          my $data=shift;
          my $return=shift;
          return 0 unless($data=~s/^( *([+\-*]) *)(?=[^+\-* ])//s || $data=~s/^( *(\d+\.) *)//s);
          my $l=length($1);
          my $type=$2;
          $type=$type=~/^\d+/?'count':'plain';
          my $text='';
          my $sin=$self->{tabwidth}||4;
          while($$data ne '')
          {
            my $pos=index($$data,"\n");
            if($pos>0)
            {
              if($data=~s/^ {$l}([^\n]+?\n)//)
              { $text.=$1; }
              elsif($$data!~/^ {0,$sin}(?:\d+\.|[*\-+])/)
              {
                my $line=substr($$data,0,$pos+1,'');
                $line=~s/^ {0,$l}//;
                $text.=$line;
              }
              else
              { last; }
            }
            elsif($pos==0)
            {
              substr($$data,0,1,'');
              last;
            }
            else
            {
              $text.=$$data;
              $$data='';
              last;
            }
          }
          $$data=~s/^\n//;
          $text=~s/(^|\n)\s{0,$l}/$1/gs if($text=~/^[^\n]\n\s{$l}/);

          if($self->_is_inside() ne "list_$type")
          {
            #print $self->_is_inside()."===> list_$type\n" ;
            $self->_do_handler('end',type=>"page") if($self->_is_inside() eq 'page');
            $self->_do_handler('start',type=>"list_$type");
          }

          $self->_do_handler('start',type=>"list_entry");
          $self->_do_handler('start',type=>"page") if($text=~/\n[^\n]+(?:\n|$)/);
          $self->_do_parse($text);
          $self->_do_handler('end',type=>"page")   if($text=~/\n[^\n]+(?:\n|$)/);
          $self->_do_handler('end',type=>"list_entry");
        },
    },
    {
      name  => 'page',
      run   => sub{
          my $self=shift;
          my $data=shift;
          return 0 unless($data=~s/^\n+//s);
          my ($type,$text) = $self->_is_inside();
          if($type eq 'page' && $text)
          {
            $self->_do_handler('end',type=>"page");
            $self->_do_handler('start',type=>"page");
          }
        },
    },
    {
      name  => 'DUMMY',
      run   => sub{
          my $self=shift;
          my $data=shift;
          return 0;
        },
    },
  );

sub _do_parse
{
  my $self=shift;
  my $data=shift;
  return 0 unless(defined($data) && $data ne '');

  while($data ne '')
  {
    my $break=0;

    for my $blk (@parse_block)
    {
      if(exists($blk->{run}) && ref($blk->{run}) eq 'CODE')
      { $break=$blk->{run}->($self,\$data); }
      last if($break);
    }

    last if($break);

    # grep everything to the next possible blockstart
    for my $regexp (@determin_block)
    {
      if($data=~$regexp)
      {
        $break=1;
        last;
      }
    }

    if($break)
    {
      my $pre=substr($data,0,$-[0],'');
      $data=~s/^\n//;
      $self->_do_parse_span($pre);
      last;
    }
    # parse per line
    # should never runn.
    else
    {
      my $line='';
      if((my $pos=index($data,"\n"))>-1)
      { $line=substr($data,0,$pos+1,''); }
      else
      {
        $line=$data;
        $data='';
      }
      $self->_do_parse_span($line);
    }
  }

  return 1;
}


sub _do_parse_old
{
  my $self=shift;
  my $data=shift;
  return 0 unless(defined($data) && $data ne '');
  # we need two lines to determin content type.
  my $sin=$self->{tabwidth}||4;
  while($data ne '')
  {

    #---------------- block elements ----------------------------------#

    #html
    if($data=~m#^ *<\w+(?:\s+\w+=(?:"[^"]*"|'[^']*'|\S+))*\s*/?>#s)
    {
      my $html='';
      ($data,$html)=$self->_cutout_html($data);
      $html=$self->_restore_quote($html);
      my $restart_page=0;
      if($self->_is_inside() eq 'page')
      {
        $self->_do_handler('end',type=>"page");
        $restart_page=1;
      }
      $self->_do_handler('start',type=>"html");
      $self->_do_handler('text',text=>$html);
      $self->_do_handler('end',type=>"html");
      $self->_do_handler('start',type=>"page") if($restart_page);
    }

    # h1/h2 Setext
    elsif($data =~s/^ *([^\n]+?) *\n *(=+|\-+) *(?:\n|$)//)
    {
      my $txt=$1;
      my $type='h1';
      $type='h2' if(index($2,'-')>=0);
      $txt=~s/^\s+//;
      $txt=~s/\s+$//;
      $self->_do_handler('end',type=>"page");
      $self->_do_handler('start',type=>$type);
      $self->_do_parse($txt);
      $self->_do_handler('end',type=>$type);
      $self->_do_handler('start',type=>"page");
    }

    # h1 .. h6 atx
    elsif($data=~s/^ *(#+) *(.+?) *#* *(?:\n|$)//)
    {
      my $p=length($1);
      $p=6 if($p>6);
      $self->_do_handler('end',type=>"page");
      $self->_do_handler('start',type=>"h$p");
      $self->_do_parse($2);
      $self->_do_handler('end',type=>"h$p");
      $self->_do_handler('start',type=>"page");
    }

    # quote and end when no more ">"
    # quote and end when "\n\n"
    elsif($data=~/^>( *)/)
    {
      my $l=length($1);
      $self->_do_handler('end',type=>"page");
      $self->_do_handler('start',type=>"quote");
      $self->_do_handler('start',type=>"page");
      my $text='';
      while($data ne '')
      {
        my $pos=index($data,"\n");
        if($pos>0)
        {
          my $line=substr($data,0,$pos+1,'');
          $line=~s/^> {0,$l}//;
          $text.=$line;
        }
        elsif($pos==0)
        {
          substr($data,0,1,'');
          last;
        }
        else
        {
          $text.=$data;
          $data='';
          last;
        }
      }
      $self->_do_parse($text);
      $self->_do_handler('end',type=>"page");
      $self->_do_handler('end',type=>"quote");
      $self->_do_handler('start',type=>"page");
    }

    # code
    elsif($data=~/^( {$sin,})/)
    {
      my $s=$1;
      my $code='';
      $code.="$1\n" while($data=~s/^$s(.*)(\n|$)//);
      $code=$self->_restore_quote($code);
      $self->_do_handler('start',type=>"code");
      $self->_do_handler('text',text=>$code);
      $self->_do_handler('end',type=>"code");
    }

    # hr
    elsif($data=~s/^(?:\- ?){3,}\s*(?:\n|$)// || $data=~s/^(?:\* ?){3,}\s*(?:\n|$)//)
    {
      $self->_do_handler('end',type=>"page");
      $self->_do_handler('start',type=>"hr");
      $self->_do_handler('end',type=>"hr");
      $self->_do_handler('start',type=>"page");
    }

    # list
    elsif($data=~s/^( *([+\-*]) *)(?=[^+\-* ])//s || $data=~s/^( *(\d+\.) *)//s)
    {
      my $l=length($1);
      my $type=$2;
      $type=$type=~/^\d+/?'count':'plain';
      my $text='';
      while($data ne '')
      {
        my $pos=index($data,"\n");
        if($pos>0)
        {
          if($data=~s/^ {$l}([^\n]+?\n)//)
          { $text.=$1; }
          elsif($data!~/^ {0,$sin}(?:\d+\.|[*\-+])/)
          {
            my $line=substr($data,0,$pos+1,'');
            $line=~s/^ {0,$l}//;
            $text.=$line;
          }
          else
          { last; }
        }
        elsif($pos==0)
        {
          substr($data,0,1,'');
          last;
        }
        else
        {
          $text.=$data;
          $data='';
          last;
        }
      }
      $data=~s/^\n//;
      $text=~s/(^|\n)\s{0,$l}/$1/gs if($text=~/^[^\n]\n\s{$l}/);

      if($self->_is_inside() ne "list_$type")
      {
        #print $self->_is_inside()."===> list_$type\n" ;
        $self->_do_handler('end',type=>"page") if($self->_is_inside() eq 'page');
        $self->_do_handler('start',type=>"list_$type");
      }

      $self->_do_handler('start',type=>"list_entry");
      $self->_do_handler('start',type=>"page") if($text=~/\n[^\n]+(?:\n|$)/);
      $self->_do_parse($text);
      $self->_do_handler('end',type=>"page")   if($text=~/\n[^\n]+(?:\n|$)/);
      $self->_do_handler('end',type=>"list_entry");
    }

    # page end/start
    elsif($data=~s/^\n+//s)
    {
      my ($type,$text) = $self->_is_inside();
      if($type eq 'page' && $text)
      {
        $self->_do_handler('end',type=>"page");
        $self->_do_handler('start',type=>"page");
      }
    }

    #---------------------- span elements -----------------------------#

    # grep everything to the next possible blockstart
    #      page end               ol                    html              quote/ul/hr/h1..6                 h1/h2              last block (span)
    elsif($data=~/\n{2,}/ || $data=~/\n\d+\./ || $data=~/\n *<\w+/ || $data=~/(?:\n[\-*>+ #]|$)/ || $data=~/\n[^\n]\n[-=]+/ || $data=~/[^\n]+$/s)
    {
      my $pre=substr($data,0,$-[0],'');
      $data=~s/^\n//;
      $self->_do_parse_span($pre);
    }

    # parse per line
    # should never runn.
    else
    {
      my $line='';
      if((my $pos=index($data,"\n"))>-1)
      { $line=substr($data,0,$pos+1,''); }
      else
      {
        $line=$data;
        $data='';
      }
      $self->_do_parse_span($line);
    }
  }
  return 1;
}

my @parse_span=(
    {
      name  => 'break',
      run   => sub{
          my $self=shift;
          my $data=shift;
          return 0 unless($line=~s/^(.*?) {2,}(?=\n|$)//s);
          $self->_do_parse_span($1);
          $self->_do_handler('start',type=>"break");
          $self->_do_handler('end',type=>"break");
          return 1;
        },
    },
    {
      name  => 'DUMMY',
      run   => sub{
          my $self=shift;
          my $data=shift;
          return 0;
        },
    },
    {
      name  => 'DUMMY',
      run   => sub{
          my $self=shift;
          my $data=shift;
          return 0;
        },
    },
  );

sub _do_parse_span_new
{
  my $self=shift;
  my $line=shift;
  #print "LINE: |$line|\n";
  return 0 unless(defined($line) && $line ne '');
  while($line ne '')
  {
    my $done=0;
    for my $span (@parse_span)
    {
      if(exists($span->{run}) && ref($span->{run}) eq 'CODE')
      { $done=$span->{run}->($self,\$line); }
      last if($done);
    }

    last if($done);

    # simple text line
    $line=$self->_unquote($line);
    $self->_do_handler('text',text=>$line);
    $line='';
  }

  return 1;
}

sub _do_parse_span
{
  my $self=shift;
  my $line=shift;
  #print "LINE: |$line|\n";
  return 0 unless(defined($line) && $line ne '');
  while($line ne '')
  {

    # break
    if($line=~s/^(.*?) {2,}(?=\n|$)//s)
    {
      $self->_do_parse_span($1);
      $self->_do_handler('start',type=>"break");
      $self->_do_handler('end',type=>"break");
    }

    # find possible span elemts
    elsif($line=~s/^(.*?)([\!\[*_`<])/$2/s)
    {
      my $type=$2;
      my $txt=$self->_unquote($1);
      $self->_do_handler('text',text=>$txt);

      # images
      if($type eq '!')
      {
        my $linkdef_ids=$self->{linkdef_ids};

        # image
        if($line=~s/^\!\[([^\]]+)\] *\((.+?)(?:\s+"([^"]+)")?\)//s)
        {
          my $alt=$1;
          my $url=$2;
          my $title=$3 || '';
          $alt=$self->_unquote($alt);
          $title=$self->_unquote($title);
          $url=~s/^<(.*)>$/$1/ if($url);
          $self->_do_handler('start',type=>"image",title=>$title,alt=>$alt,url=>$url);
          $self->_do_handler('end',type=>"image",title=>$title,alt=>$alt,url=>$url);
        }

        # imageref
        elsif($line=~s/^\!\[(.+?)\]\s*\[($linkdef_ids)\]//s)
        {
          my $id=$2;
          my $alt=$1;
          $id=~s/\s+/ /gs;
          my $pid=$self->_unquote($id);
          my %attr=%{$self->{link_defs}->{$id}};
          $attr{type}="imageref";
          $attr{id}=$pid;
          $attr{alt}=$self->_unquote($alt);
          $self->_do_handler('start',%attr);
          $self->_do_handler('end',%attr);
        }
        else
        {
          substr($line,0,1,'');
          $self->_do_handler('text',text=>$type);
        }
      }

      # links
      elsif($type eq '[')
      {
        my $linkdef_ids=$self->{linkdef_ids};

        # link
        if($line=~s/^\[([^\]]+?)\]\s*\(([^()]+?)\s+"(.+?)"\s*\)//s || $line=~s/^\[([^\]]+?)\]\s*\(([^)]+?)\)//s)
        {
          my $text=$1;
          my $url=$2;
          my $title=$3 || '';
          $url=~s/^<(.*)>$/$1/ if($url);
          $self->_do_handler('start',type=>"link",title=>$title,url=>$url);
          $self->_do_parse_span($text);
          $self->_do_handler('end',type=>"link",title=>$title,url=>$url);
        }

        # linkdef
        elsif($line=~s/^$grep_link_def//s)
        {
          my $id=$1;
          my $url=$2;
          my $title=$3||$id;
          $url=~s/^<(.*)>$/$1/ if($url);
          unless($title=~s/^(["'])(.+?)\1$/$2/)
          { $title=~s/^\((.+?)\)$/$1/; }
          $id=~s/\s+/ /gs;

          $self->add_link_def($id,$url,$title);

          my $pid=$self->_unquote($id);
          $self->_do_handler('start',type=>"linkdef",id=>$pid,title=>$title,url=>$url);
          $self->_do_handler('end',type=>"linkdef",id=>$pid,title=>$title,url=>$url);
        }

        # linkref
        elsif($line=~s/^\[(.+?)\]\s*\[($linkdef_ids)\]//s || $line=~s/(^)\[($linkdef_ids)\](?:\s*\[\])?//s)
        {
          my $id=$2;
          my $text=$1 || '';
          $id=~s/\s+/ /gs;
          my $pid=$self->_unquote($id);
          $self->_do_handler('start',type=>"linkref",id=>$pid,%{$self->{link_defs}->{$id}});
          if($text)
          {
            $text=$self->_unquote($text);
            $self->_do_handler('text',text=>$text);
          }
          $self->_do_handler('end',type=>"linkref",id=>$pid,%{$self->{link_defs}->{$id}});
        }

        else
        {
          substr($line,0,1,'');
          $self->_do_handler('text',text=>$type);
        }
      }

      # strong/em
      elsif($type eq '*' || $type eq '_')
      {
        #** text **
        #__ text __
        if($line=~s/^(\*{1,3})([^*]+)\1//s || $line=~s/^(_{1,3})([^_]+)\1//s)
        {
          if(length($1)==3)
          {
            my $text=$2;
            $self->_do_handler('start',type=>'strong');
            $self->_do_handler('start',type=>'em');
            $self->_do_parse_span($text);
            $self->_do_handler('end',type=>'em');
            $self->_do_handler('end',type=>'strong');
          }
          else
          {
            my $type=length($1)>1?'strong':'em';
            my $text=$2;
            $self->_do_handler('start',type=>$type);
            $self->_do_parse_span($text);
            $self->_do_handler('end',type=>$type);
          }
        }
        else
        {
          substr($line,0,1,'');
          $self->_do_handler('text',text=>$type);
        }
      }

      # inlinecode
      elsif($type eq '`')
      {
        # `code`
        # ``code``
        if($line=~s/^(`{1,2})(.+?)\1//s)
        {
          my $l=length($1)-1;
          my $code=$self->_restore_quote($2);
          $code=~s/\`/`/gs unless($l);
          $self->_do_handler('start',type=>'spancode');
          $self->_do_handler('text',text=>$code);
          $self->_do_handler('end',type=>'spancode');
        }
        else
        {
          substr($line,0,1,'');
          $self->_do_handler('text',text=>$type);
        }
      }

      # html/url
      elsif($type eq '<')
      {
        # comments
        if($line=~s/^<!--(.*?)-->//s)
        {
           my $txt=$self->_restore_quote($1);
           $self->_do_handler('start',type=>'comment');
           $self->_do_parse($txt);
           $self->_do_handler('end',type=>'comment');
        }

        # inline html
        elsif($line=~m#^(<\w+(?:\s+\w+=(?:"[^"]*"|'[^']*'|\S+))*\s*/?>)#s)
        {
          my $html='';
          ($line,$html)=$self->_cutout_html($line);
          $html=$self->_restore_quote($html);
          $self->_do_handler('start',type=>"html");
          $self->_do_handler('text',text=>$html);
          $self->_do_handler('end',type=>"html");
        }
        # simple url
        elsif($line=~s#^<(http://[^>]+|[^@>]+@[^>]+)>##s)
        {
          $self->_do_handler('start',type=>"link",url=>$1);
          $self->_do_handler('text',text=>$1);
          $self->_do_handler('end',type=>"link",url=>$1);
        }
        else
        {
          substr($line,0,1,'');
          $self->_do_handler('text',text=>$type);
        }
      }
    }

    # simple text line
    else
    {
      $line=$self->_unquote($line);
      $self->_do_handler('text',text=>$line);
      $line='';
    }
  }
  return 1;
}

sub _make_link_def_regexp
{
  my $self=shift;
  # create new list for link ids
  # special: all spaces and newlines in IDs should handled as same
  my @list;
  for (keys(%{$self->{link_defs}}))
  {
    my $o;
    my @l=split(/\s+/,$_);
    my $s=pop(@l);
    $o=$o? qr/$o\Q$_\E\s+/ : qr/\Q$_\E\s+/ for(@l);
    $o=$o? qr/$o\Q$s\E/ : qr/\Q$s\E/;
    push(@list,$o)
  }
  $self->{linkdef_ids}=join('|',@list);
  #print "linkdef_ids ==>> $self->{linkdef_ids}\n";
}

sub _cutout_html
{
  my $self=shift;
  my $data=shift;
  my $html='';
  my @deep;
  while($data=~m#<(/?)(\w+)(?:\s+\w+\s*=(?:\s*"[^"]*"|\s*'[^']*'|[^'"/\s]+))*\s*(/?)>#s)
  {
    $html.=substr($data,0,$+[0],'');
    unless($3)
    {
      my $tag=$2;
      if($1)
      {
        shift(@deep) while(@deep && $deep[0] ne $tag);
        shift(@deep);
      }
      else
      { unshift(@deep,$tag); }
    }

    last unless(@deep);
  }

  if(@deep && $self->{grep_exessive})
  {
    $html.=$data;
    $data='';
  }

  return($data,$html);
}

sub _restore_quote
{
  my $self=shift;
  my $txt=shift || '';
  $txt=~s/(&quote\d+;)/\\$self->{quotes}->{$1}/gs;
  return $txt;
}

sub _unquote
{
  my $self=shift;
  my $txt=shift || '';
  #my @l=%{$self->{quotes}}; warn("@l\n");
  $txt=~s/(&quote\d+;)/$self->{quotes}->{$1}/gs;
  return $txt;
}

sub _is_inside
{
  my $self=shift;
  return '' unless(@{$self->{deep}});
  if(@_ && defined($_[0]))
  {
    if(ref($_[0]) eq 'Regexp')
    {
      for(reverse @{$self->{deep}})
      { return $_->{type} if($_->{type}=~$_[0]); }
    }
    else
    {
      for(reverse @{$self->{deep}})
      { return $_->{type} if($_->{type} eq $_[0]); }
    }
    return '';
  }

  return ($self->{deep}->[-1]->{type},$self->{deep}->[-1]->{text}) if(wantarray);
  return $self->{deep}->[-1]->{type};
}

sub _do_handler
{
  my $self=shift;
  my $type=shift;
  my %opts=@_;
  $opts{self}=$self;
  for(qw(type url title alt id))
  { $opts{$_}='' unless(defined($opts{$_})); }

  return 0 unless(__is_allowed_type($type));
  if($type eq 'start')
  {
    if(@{$self->{deep}})
    {
      unless(exists($self->{deep}->[-1]->{text}))
      { $self->{deep}->[-1]->{text}=[]; }
      push(@{$self->{deep}->[-1]->{text}},'') ;
    }
    my $text=$opts{text};
    delete($opts{text});

    push(@{$self->{deep}},{%opts});
    $self->start(@opts{qw(type url title alt)});
    $self->_process_handler($type,%opts);
    $self->_do_handler('text',text=>$text) if(defined($text));
  }
  elsif($type eq 'end')
  {
    while(@{$self->{deep}})
    {
      my $text=$opts{text};
      delete($opts{text});

      my $type_now=$self->{deep}->[-1]->{type};
#      print "TEST  $type_now <> $opts{type}\n";
      if($type_now eq $opts{type})
      {
        $self->_do_handler('text',text=>$text) if(defined($text));
        my $p=pop(@{$self->{deep}});

        while(my ($k,$v)=each(%$p))
        { $opts{$k}=$v unless($opts{$k}); }

        $self->_process_handler('end',%opts);
        $self->end(@opts{qw(type text url title alt)});
        last;
      }
      my %o=%{pop(@{$self->{deep}})};
      $self->end(@o{qw(type text url title alt)});
      $self->_process_handler('end',%o);
    }
  }
  elsif($type eq 'text')
  {
    $opts{type}=$self->{deep}->[-1]->{type} unless($opts{type});

    my $inside=$self->_is_inside();
    if($inside eq 'list_plain' || $inside eq 'list_count')
    {
      $self->_do_handler('end',type=>$inside);
      $self->_do_handler('start',type=>'page');
    }

    #$self->_do_handler('start',type=>'page') unless(@{$self->{deep}});

    if(@{$self->{deep}})
    {
      unless(exists($self->{deep}->[-1]->{text}))
      { $self->{deep}->[-1]->{text}=['']; }

      if(@{$self->{deep}->[-1]->{text}})
      { $self->{deep}->[-1]->{text}->[-1].=$opts{text}; }
      else
      { push(@{$self->{deep}->[-1]->{text}},$opts{text} || ''); }
      #$opts{text}=$self->{deep}->[-1]->{text};
    }
    $self->text(@opts{qw(type text)});
    $self->_process_handler('text',%opts);
  }
  return 1;
}

sub _process_handler
{
  my $self=shift;
  my $type=shift;
  my %opts=@_;
  $opts{self}=$self;
  return 0 unless(__is_allowed_type($type));
  if($self->{handler}->{$type} && @{$self->{handler}->{$type}})
  {
    my $pos=$#{$self->{handler}->{$type}};
    for(reverse @{$self->{handler}->{$type}})
    {
      next unless($_);
      my ($code,@param)=@$_;
      my $ret=$code->(@opts{@param});
      $self->_remove_handler($type,$pos) if($ret<=0);
      $pos--;
    }
  }
  return 1;
}

=item add_link_def($id,$url,$title)

add a link definition.

=cut

sub add_link_def
{
  my $self=shift;
  my ($id,$url,$title)=@_;
  return 0 unless($id && $id);
  $title='' unless(defined($title));
  unless(exists($self->{link_defs}->{$id}))
  {
    $self->{link_defs}->{$id}={url=>$url,title=>$title};
    return 1;
    $self->_make_link_def_regexp();
  }
  return 0;
}

#sub utf8
#{
#  my $self=shift;
#  if(@_)
#  {
#    $self->{utf8}=0;
#    $self->{utf8}=1 if($_[1]);
#  }
#  return $self->{utf8};
#}

=item add_handler($type,$code_ref,$param)

Add an handler.

returnes the name of the handler in the Stack

whenever an formating-tag will be started, or text added,
or finished, the specified handler will be executed

Allowed types: start, text, end

$param is an string of comma separated valued,
who specifed witch values will be pased to the codereference.

Allowed:

 self  - the parser object
 text  - the text inside the tag (empty in "start", a String in "text" and an array in "end")
 url   - an URL set by links or images
 alt   - alternatibe text for images
 title - title of an link or image
 id    - id of a link definition
 type  - type of the Tag
         The following strings of types will be pasted to the coderefence:
          list_plain - a dotted list
          list_count - a counted list
          list_entry - list entry
          h[1..6]    - heads 1 .. 6 (param: self text)
          hr         - line         (possible params: self)
          link       - link         (possible params: self text url title)
          linkref    - link         (possible params: self text url title id)
          link_def   - link         (possible params: self url title id)
          image      - image        (possible params: self url title alt)
          imageref   - image        (possible params: self url title alt id)
          page       - page         (possible params: self text)
          code       - codeblock    (possible params: self text)
          inlinecode - inlinecode   (possible params: self text)
          html       - htmlblock    (possible params: self text)
          inlinehtml - inlinehtml   (possible params: self text)
          quote      - quoted text  (possible params: self text)
          strong     - strog        (possible params: self text)
          em         - em           (possible params: self text)
          break      - linebreak    (possible params: self)
          comment    - html-comment (possible params: self text)

the code refence has to return true '1'.
If false (0) returned the handler will be ermoved from the handlerstack

the last added handler will be executed first.

=cut

sub add_handler
{
  my $self=shift;
  my $type=shift;
  my $code=shift;
  my $param=shift;
  $param="self,type,text,url,alt" unless($param);
  return undef unless(__is_allowed_type($type));
  return undef unless($code && ref($code) eq 'CODE');

  my @pp=split(/\s*,\s*|\s+/,$param);
  for(@pp)
  { return 0 unless(__is_allowed_param($_)); }

  push(@{$self->{handler}->{$type}},[$code,@pp]);
  return $type.$#{$self->{handler}->{$type}};
}

=item del_handler($id)

delete handler by id.

=cut

sub del_handler
{
  my $self=shift;
  my $dd=shift;
  return 0 unless($dd);
  my ($type,$id)=$dd=~/^(.+?)(\d+)$/;
  return _remove_handler($type,$id);
}

sub _remove_handler
{
  my $self=shift;
  my $type=shift;
  my $id=shift;
  return 0 unless(__is_allowed_type($type));
  return 0 if(defined($id) && $id > $#{$self->{handler}->{$type}} || $id < 0);
  $self->{handler}->{$type}->[$id]=undef;
}

########################################################################

sub __is_allowed_type
{
  return 0 unless($_[0]);
  my $c=grep{$_ eq $_[0]}qw(text start end);
  return  $c > 0;
}

sub __is_allowed_param
{
  return 0 unless($_[0]);
  my $c=grep{$_ eq $_[0]}qw(self type text url alt title id);
  return  $c > 0;
}

1;

=back

=cut

=head1 BUGS

To prevent doubleparsing the text the linkdefinitions are captured by an regexp when the parsing beginns.
but there is no easy way to find _all_ linkdefinitions without doubleparsing,
so it's possible not all link-refences will be found.

=head1 TODO

- optional doubeparsing to find all linkdefinitions

- optional warnings

- implement an easy way to extend the parser

=head1 See also

L<Text::Markdown::TreeBuilder>

=head1 AUTHOR

Tobias Grönhagen <markdown@topeg.de>

=head1 VERSION

Version 0.1 1 January 2011

=head1 COPYRIGHT

Copyright (c) Tobias Grönhagen. All rights reserved.

=head1 LICENSE

This program is free software; you can redistribute it
and/or modify it under the same terms and conditions as
Perl itself.

This means that you can, at your option, redistribute it and/or modify it under
either the terms the GNU Public License (GPL) version 2 or later, or under the
Perl Artistic License.

See http://dev.perl.org/licenses/

=head1 DISCLAIMER

THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS
OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE.

Use of this software in any way or in any form, source or binary,
is not allowed in any country which prohibits disclaimers of any
implied warranties of merchantability or fitness for a particular
purpose or any disclaimers of a similar nature.

IN NO EVENT SHALL I BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
SPECIAL, INCIDENTAL,  OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
USE OF THIS SOFTWARE AND ITS DOCUMENTATION (INCLUDING, BUT NOT
LIMITED TO, LOST PROFITS) EVEN IF I HAVE BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE

=cut
